#include <PalmOS.h>
#undef vchrSonyMin
#undef vchrSonyMax

#include <SonyCLIE.h> // for Clie related
#include <vga.h> // For HandEra330 related
#include <HsExt.h> // for Visor/Treo related
#include <PalmChars.h> // for 5Way Navigator
#include "MyUtility.h"

static FormType *saved_form;
UInt32 OSVersion;
Boolean IsColorOS; // true is OS is later than 3.5
Boolean ExtMediaSupported;  // True is has VFSManager
Boolean JogDialSupported; // True if has JogDial

MemPtr GetObjectPtrFromID(UInt16 objectID)
{
	FormType *frmP;

	frmP = FrmGetActiveForm();
	return (FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID)));
}

UInt16 GetObjectIndexFromID(UInt16 objectID)
{
	FormType *frmP;

	frmP = FrmGetActiveForm();
	return ( FrmGetObjectIndex(frmP, objectID) );
}

UInt16 GetObjectIDFromIndex(UInt16 objectIndex)
{
	FormType *frmP;

	frmP = FrmGetActiveForm();
	return ( FrmGetObjectId(frmP, objectIndex) );
}

MemPtr GetObjectPtrFromIndex(UInt16 objectIndex)
{
	FormType *frmP;

	frmP = FrmGetActiveForm();
	return ( FrmGetObjectPtr(frmP, objectIndex) );
}

UInt16 GetObjectIDFromPtr( MemPtr objectPtr)
{
	return GetObjectIDFromIndex( GetObjectIndexFromPtr( objectPtr));
}

UInt16 GetObjectIndexFromPtr(MemPtr objectPtr)
{
	FormType *frmP = FrmGetActiveForm();
	UInt16 index;
	UInt16 objIndex = frmInvalidObjectId;
	UInt16 numObjects = FrmGetNumberOfObjects(frmP);

	for (index = 0; index < numObjects; index++) {
		if (FrmGetObjectPtr(frmP, index) == objectPtr) {
			// Found it
			objIndex = index;
			break;
		}
	}
	
	return objIndex;
}

void
MyLibPresetPopTrigger( UInt16 triggerID, UInt16 listID, UInt16 selection)
{
	LstSetSelection ( GetObjectPtr( listID), selection);
	CtlSetLabel( GetObjectPtr( triggerID), LstGetSelectionText( GetObjectPtr( listID), selection)) ; 
}


FormType *ShowWaitWindow(void)
{
	FormType *form;
	MachineFunctionType func;
	GetMachineFunctionality(&func);
	
	saved_form = FrmGetActiveForm();
	form = FrmInitForm( WaitForm);
//	if (  func.supportHandEraHiRes){
//		VgaFormModify( form, vgaFormModify160To240 );
//	}
//	FrmSetEventHandler(form,DummyHandleEvent);
	FrmSetActiveForm( form);
	FrmDrawForm( form);
	return form;
}

void
EraseWaitWindow( FormType *form)
{
	FrmEraseForm( form);
	FrmDeleteForm(form);
	FrmSetActiveForm( saved_form);
}

void ShowObject( int id)
{
	FormType *form=FrmGetActiveForm();
	FrmShowObject( form ,FrmGetObjectIndex( form, id));
}

void HideObject( int id)
{
	FormType *form=FrmGetActiveForm();
	FrmHideObject( form ,FrmGetObjectIndex( form, id));
}

void SetTextOnField(
		FieldType* field,
		Char* str)
{
	MemHandle strHand;
	MemHandle orgHandle;
	
	orgHandle = FldGetTextHandle( field);
//	FldFreeMemory(field);
	
	{
		UInt32 strLen = StrLen(str) + 1;
		strHand = MemHandleNew(strLen);
		ErrFatalDisplayIf(strHand == 0, "No memory.");
		StrCopy(MemHandleLock(strHand), str);
		MemHandleUnlock(strHand);
	}
	
	FldSetTextHandle(field, strHand);
	
	if ( orgHandle)
		MemHandleFree( orgHandle);
}

Boolean
SearchForFilesInDirectory(UInt16 volumeRefNum,FileRef *file, char *path, UInt32 *iterator,FileInfoType *info)
{
	Err err;
//	FileRef file;
//	char file_name[33];
	Boolean result = false;
	
// If first time, let open Directory
	if (*iterator == 0){

		err= VFSFileOpen(volumeRefNum, path, vfsModeRead, file);
		if ( err != errNone){
			return false;
		}

	} else if ( *iterator == expIteratorStop){
		return false;
	}	
	
// Let's search for the File

	switch ( VFSDirEntryEnumerate( *file, iterator, info)){
		case errNone:
			if ( *iterator == expIteratorStop ){
				err = VFSFileClose( *file);
				ErrFatalDisplayIf( err != errNone, "Error in VFSFileClose");
			}
			result =  true;
			break;

		default:
			VFSFileClose( *file);
			result = false;
			break;
	}	

	return result;
}

//
// Check if This Palm has ability to handle MemoryStick(storage type) by checking existance of VFS Manager
//
// Return 
// false : No VFS Manager . Can't handle Memory Stick
// true: VFS Manager exists. Can handle Memory Stick
//
Boolean
CheckMSFunctionality(void)
{
	UInt32 version;
	Boolean result;
	
// Check if VFS Manager Exist
	if (FtrGet(sysFileCVFSMgr, vfsFtrIDVersion, &version) == 0) { // Feature Found.
		result = true;
	} else { // Not found VFS Manager is not found
		result = false; 
	}
	
	return result;
}

UInt16 MSGetNumVolumes()
{
	int i;
	UInt32 volumeIterator;
	UInt16 volumeRefNum=0;
	
	for (volumeIterator =expIteratorStart,  i = 0; ; ){
		if ( VFSVolumeEnumerate(&volumeRefNum, &volumeIterator) != errNone)
			break;
		i ++;
		if ( volumeIterator == expIteratorStop)
			break;
	}
	
	return i;
}


UInt16 MSGetVolumeRefNum( int number)
{
	int i;
	UInt32 volumeIterator;
	UInt16 volumeRefNum=0;
	
	for ( i =expIteratorStart, volumeIterator = 0;i<= number ; i++)
		if ( VFSVolumeEnumerate(&volumeRefNum, &volumeIterator) != errNone)
			break;
			
	return volumeRefNum;
}

Err MyLibGetVolumeList(MemHandle vol_name_lists, UInt16 *num_volumes, MemHandle ref_num_lists )
{
	int i;
	UInt32 volumeIterator;
	UInt16 volumeRefNum=0;
	UInt16 *ref_numP;
	char *volName;
	char **vol_name_listP;
	Err err;
	
	volName = MemPtrNew(256);
	err = errNone;

	for (volumeIterator =expIteratorStart,  i = 0;( volumeIterator != expIteratorStop) && ( i < 256) ; ){
		if ( (err = VFSVolumeEnumerate(&volumeRefNum, &volumeIterator)) != errNone){
			goto EXIT;
		}

		i ++;

// Set volume ref num
		MemHandleResize( ref_num_lists, sizeof(UInt16)*i);
		ref_numP = MemHandleLock( ref_num_lists);
		*(ref_numP + (i-1) ) = volumeRefNum;
		MemHandleUnlock( ref_num_lists);

// Get volume name	
		err = VFSVolumeGetLabel(  volumeRefNum ,volName, 256);
		if ( err != errNone){
			goto EXIT;
		}

// Set volume ref num
		MemHandleResize( vol_name_lists, sizeof(MemPtr)*i);
		vol_name_listP = MemHandleLock(  vol_name_lists );
		*(vol_name_listP + (i-1) ) = MemPtrNew( StrLen( volName) +1);
		StrCopy( *(vol_name_listP + (i-1) )  ,volName );
		MemHandleUnlock( vol_name_lists);		
	} 
	
EXIT: 
	if (volName)
		MemPtrFree( volName);
			
	*num_volumes = i;	
	return errNone ;
}

Err MyLibMakeVFSFileList(UInt16 volRefNum, char *dir_path, UInt32 target_creator,UInt32 target_type, MemHandle listH, UInt16 *num_files)
{
	UInt32 iterator = 0;
	FileInfoType finfo;
	FileRef dir, file;
	char *filename_buf = MemPtrNew(256);
	char *full_path = MemPtrNew(256);
	char db_name[33];
	UInt32 type, creator;
	UInt32 files_len=0;
	Err err = errNone;
	
	*num_files = 0;

	finfo.nameP = filename_buf;
	finfo.nameBufLen  =256;
	
	while ( true ){
		if (! SearchForFilesInDirectory( volRefNum, &dir, dir_path, &iterator,&finfo)){
			break;
		}
		
		StrCopy( full_path, dir_path);
		StrCat( full_path , finfo.nameP);
		if (VFSFileOpen( volRefNum, full_path, vfsModeRead, &file) == errNone){
			if (VFSFileDBInfo( file ,db_name,0, 0,0, 0,0,0,0,0,&type,&creator,0) == errNone){
				if ( ( (target_type == 0 ) || ( target_type == type) ) && ( (target_creator == 0 ) || ( target_creator == creator ) ) ){
// Found!!!
					char *buf;
					
					MemHandleResize( listH, files_len + StrLen( filename_buf) + 1);
					buf = MemHandleLock( listH);
					StrCopy( buf+ files_len, filename_buf);
					MemHandleUnlock( listH);
					
					files_len += StrLen( filename_buf)+1;
					
					(*num_files)++;					
				}
			}
			VFSFileClose( file);
		}		
	}	

EXIT:
	MemPtrFree( filename_buf);
	MemPtrFree( full_path);

	return err;
}

Err MyLibGetDBNameFromMemoryCard( UInt16 volRefNum,char * full_path ,char *db_name)
{
	Err err = errNone;
	FileRef file;
	
	err = VFSFileOpen( volRefNum, full_path, vfsModeRead , &file);
	if ( err == errNone){
		err = VFSFileDBInfo( file ,db_name,0, 0,0, 0,0,0,0,0,0,0,0);
		VFSFileClose( file);
	}

	return err;
}	

void CheckCreate( UInt16 volRefNum, char *path)
{
	Err err;
	FileRef dir;
	UInt32 attr;
	
	err = VFSFileOpen( volRefNum, path, vfsModeRead , &dir);
	if ( err == errNone){
		err =  VFSFileGetAttributes(dir, &attr);
		if ( attr & vfsFileAttrDirectory == 0){ // It's  not directory. Let's erase it
			VFSFileClose( dir);
			VFSFileDelete( volRefNum, path);
		} else {
			VFSFileClose( dir);
			return;
		}
	}

	VFSDirCreate( volRefNum, path);	
	return;
}

extern Err MyLibVolumeWriteProteced( UInt16 volRefNum,Boolean *protected)
{
	VolumeInfoType info;
	Err err;
	
	err = VFSVolumeInfo(volRefNum, &info);
	if ((info.attributes & vfsVolumeAttrReadOnly) != 0 )
		*protected  = true;
	else
		*protected  = false;

	return err;
}

Boolean IsSonyDevice(void)
{
	UInt32 value;
	
	FtrGet( 'psys', sysFtrNumOEMCompanyID, &value);
	if ( value == 'sony')
		return true;
	else
		return false;
}

MemPtr GetObjectPtr(Int16 objectID)
{
	FormPtr frm;
	
	frm = FrmGetActiveForm();
	return(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, objectID)));
}

// Check the OS  version and return FALSE is os is not supported.
// and Check is this os supports Color Functions. It's returned in global IsColorOS
Boolean OsVersionCheck(void)
{
	UInt32 romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < sysMakeROMVersion(3,0,0,sysROMStageRelease,0)){
		FrmAlert (RomIncompatibleAlert);
		
		// Pilot 1.0 will continuously relaunch this app unless we switch to 
		// another safe one.
		if (romVersion < sysMakeROMVersion(2,0,0,sysROMStageRelease,0)){
			AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
		}
		
		return (0);
	}

// Check OS supports Color Drawing
	IsColorOS = romVersion >= sysMakeROMVersion(3,5,0,sysROMStageRelease,0);
	OSVersion = romVersion;

// Check VFS Manager	
	if (OSVersion >= 0x03500000) { // check driver existence 
		UInt32 version;
		if (FtrGet(sysFileCVFSMgr, vfsFtrIDVersion, &version) == 0) {
			ExtMediaSupported = true;
		} else {
			ExtMediaSupported = false;
		}
	}

// Check Jog Dial existance( Not ideal way, but there's none other than this way...)
	JogDialSupported = IsSonyDevice();
	
	return (1);
}

// Check the OS  version and return FALSE is os is not supported.
// and Check is this os supports Color Functions. It's returned in global IsColorOS
void
 GetMachineFunctionality(MachineFunctionType *func)
{
	UInt32 romVersion;
	Err error;
	UInt32 hal,company, device;	
// Set all to false

/*	func->supportDynamicUI = false;
	func->supportBeam = false;
	func->supportFileStream = false;
	func->supportColorDrawing = false;
	func->supportDirectColor = false;
	func->colorScreen = false;
	func->supportHiResolution = false;
	func->supportHandEraHiRes = false;
	func->hasJogDial = false;
	func->hasBackButton = false;
	func->hasJogAssist = false;
	func->supportExternalMedia = false;
	func->supportHandEraHiRes = false;
	func->hasHandEraJog = false; 
	func->vfsManagerVersion = 0;
	func->osVersion = 0;
	func->colorCapability = 0;
*/

// Get Company, Device and Hal
// Use these ONLY WHEN there's no appropriate method to determine device functionality

	FtrGet(sysFtrCreator, sysFtrNumOEMCompanyID, &company);
 	FtrGet(sysFtrCreator, sysFtrNumOEMDeviceID, &device);
 	FtrGet(sysFtrCreator, sysFtrNumOEMHALID, &hal);

	MemSet( (MemPtr)func, sizeof(MachineFunctionType), 0);
	
	func->structureVersion = MACHINE_FUNCTION_TYPE_VERSION;

	func->hasGraffitiArea = true;
	func->nativeDensity =kCoordinatesStandard;

	
// Get Screen Extent
// Caution!! We'll adjust this for different HighRes supports(CLie,HandEra,HighDensity)
	{
		Int16 x,y;
		WinGetDisplayExtent( &x,&y);
		func->screenWidth = x;
		func->screenHeight = y;;
		func->screenWidthPixels = func->screenWidth;
		func->screenHeightPixels = func->screenHeight;
	}
	
// GET OS Version
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	func->osVersion = (romVersion & 0xffff0000 )>> 16; // LIKE 0x351 or 0x200 

// See OS3.0 and later functions				    
	if (romVersion > 0x03000000) { // OS 3.0 or later, has Dynamic UI, File Stream and Beam
		func->supportDynamicUI = true;
		func->supportBeam = true;
		func->supportFileStream = true;
	}

// Check VFS Manager	
	{
		UInt32 version;
		if (FtrGet(sysFileCVFSMgr, vfsFtrIDVersion, &version) == 0) {
//			char buf[64];
			func->supportExternalMedia = true;
			if ( version <= 0x01)
				func->vfsManagerVersion =1; // for Clie on 3.5
			else
				func->vfsManagerVersion =2; // for m500/505 on PalmOS4.0
		} else {
			func->supportExternalMedia = false;
			func->vfsManagerVersion =0; // None
		}
	}

// Check for keyboard type Treo
	{
		UInt32 dummy;
		FtrGet(hsFtrCreator , hsFtrIDTypeOfKeyboard , &dummy);

		if ( dummy & hsFtrValKeyboardJog ){
			func->hasTreoJog = true;
		}
		
		if ( dummy &  hsFtrValKeyboardQwerty ){
			func->hasGraffitiArea = false;
			func->hasTreoKeyboard = true;
		}
	}

// Check LID availablity on treo 
// There's no way to direct check. We see by Radio ability
	{
		UInt32 dummy;
		FtrGet(hsFtrCreator ,hsFtrIDRadioAttributes , &dummy);

		if ( dummy & hsFtrValRadioBuiltin ){
			func->hasTreoLid = true;
			func->hasTreoRadio = true;
		}			
	}

// Check for 5 Way Navigator on Palm Tungsten series
	{
		UInt32 version;
		Err err = FtrGet(navFtrCreator, navFtrVersion, &version);	

		if ( err == errNone){
			func->has5WayNavigator = true;
		}
	}
	
// Check for 2 Button only device like Zire
	if ( ( company == 'Palm') && ( device == 'Cubs')){ // Zire (aka m150)
		func->hasOnly2Button = true;
	}
		
	if (romVersion > 0x03500000) { // OS 3.5 or later, let's check about color
		void* procP;

// OS 3.5 has color drawing
		func->supportColorDrawing = true;

// Check has Color Screen and Get Color Bit depth capability
		error = WinScreenMode ( winScreenModeGetSupportsColor, 0, 0, 0, &(func->colorScreen) );
		error = WinScreenMode ( winScreenModeGetSupportedDepths ,0, 0, &(func->colorCapability), 0 );


// Check supports direct color
		procP = SysGetTrapAddress( sysTrapWinSetForeColorRGB);
		if (procP && procP!= SysGetTrapAddress(sysTrapSysUnimplemented)){
			func->supportDirectColor = true;
		}

// Now Check for CLIE related feature
// Check for Hi Resolution display
		{
			SonySysFtrSysInfoP sonySysFtrSysInfoP;
			Err error = 0;
			
			if ((error = FtrGet(sonySysFtrCreator, sonySysFtrNumSysInfoP,	(UInt32*)&sonySysFtrSysInfoP))) {	
// It not CLIE N700 or later. Let's check is S500/S300
				if (IsSonyDevice()){
					 func->hasJogDial = true;
				}
			}  else { // It CLIE equal or later than N700
// Has Jog Dial
				 if ( sonySysFtrSysInfoP->extn & sonySysFtrSysInfoExtnJog){
					 func->hasJogDial = true;
				}

// Has Jog Assist?
				 if ( sonySysFtrSysInfoP->extn & sonySysFtrSysInfoExtnJogAst ){
					 func->hasJogAssist = true;
				}

// If JogType is Type2 or later, has Back button( workon S320/S360? Not Checked)
				if ( sonySysFtrSysInfoP->jogType >= sonySysFtrSysInfoJogType2)
					func->hasBackButton =true;

// Support Sony type HiRes
				if (sonySysFtrSysInfoP->libr & sonySysFtrSysInfoLibrHR) { // Support HiREs
					func->supportHiResolution = true;
					func->nativeDensity = kDensityDouble;

					func->screenWidthPixels = func->screenWidth * 2;
					func->screenHeightPixels = func->screenHeight * 2;
				}

// Support Sony Type SoftSilk ?
				if (sonySysFtrSysInfoP->extn & sonySysFtrSysInfoExtnSilk ) { //
					func->supportClieSoftSilk = true;
				}

			}
		}

// Now Check for HandEra
		{
			UInt32 version;
			
			if ( FtrGet(TRGSysFtrID, TRGVgaFtrNum, &version) == 0){ // It's HandEra
				func->supportHandEraHiRes = true;
				func->nativeDensity = kCoordinatesStandard;
				func->hasHandEraJog = true; // This should be checked in another manner. But there's no good checking method so far.

//				func->screenWidth  = func->screenWidth * 2 / 3;
//				func->screenHeight = func->screenHeight * 2 / 3;

			}
		}

// Check for High Density Support
		{
			UInt32 version;
			Err err;
			
			err = FtrGet(sysFtrCreator,sysFtrNumWinVersion, &version);
			if ( version >= 4){
				UInt32 density, x, y;
				func->supportHighDensity  = true;

				WinScreenGetAttribute( winScreenDensity, &density);
				func->nativeDensity = density;
				WinScreenGetAttribute( winScreenWidth , &x);
				WinScreenGetAttribute( winScreenHeight , &y);
				func->screenWidthPixels = x;
				func->screenHeightPixels = y;

			}
		}
				
	} // End of OS3.5

// Get Screen Extent

	return ;
}


static void RemoveExtension(char * name) 
{
	int i;
// Search for extension and remove it
	i = StrLen( name);
	if ( ( i > 4) && (name[i-4] == '.'))
		name[i-4] = 0;
}


void FileNameToDatabaseName	(char * dbName, char *DocumentName) // Remove extension and shrink to 32 byte
{
	char buf[256];
	char *p = buf;

// Shrink to 32 byte
	while ( *DocumentName && ( p - buf) < 29 ){
		if ( IsKanji( (UInt8 *)DocumentName) ){
			*(p++) = *(DocumentName++);
			*(p++) = *(DocumentName++);
		} else if ( (*DocumentName == '%' ) && *(DocumentName+1)  && *(DocumentName+2) ) {
			*(p++)= HexToInt(*(DocumentName+1), *(DocumentName+2));
			DocumentName += 3;
		} else {
			*(p++) = *(DocumentName++);
		}
	}
	*(p++)='\0'; // Terminate
	
//	FrmCustomAlert( DebugAlert, buf,"Original","");				
	RemoveExtension( buf);	

//	FrmCustomAlert( DebugAlert, buf,"Extension Removed","");				
	buf[31] = 0;
//	FrmCustomAlert( DebugAlert, buf,"Truncated to 31 chars","");				

	StrCopy( dbName, buf);
//	FrmCustomAlert( DebugAlert, dbName,"Final","");				

}


int Hex1ToInt( char c)
{
	char hex[] ="0123456789ABCDEF";
	int i;
	
	if ( ( c >= 'a' ) && ( c <= 'z'))
		c += 'A' - 'a';
		
	for ( i = 0 ; i < 16 ; i++)
		if ( hex[i] == c)
			return i;
	
	return 0;
}

int HexToInt(char b1, char b2)
{
	return ( Hex1ToInt( b1 ) * 0x10 + Hex1ToInt( b2 ));	
}


static UInt8 IntToHex(int c)
{
	char hex[]="0123456789ABCDEF";
	return ( (UInt8)hex[c]);
}

static Boolean IsValidFileNameChar(UInt8 c)
{
	UInt8  valid[] = "-@()#+"; // Reduced valid chars. More to be encoded
//	UInt8  valid[] = "$'-@~`!()^#&+,;=[]";
	int i;
	
	for (i=0; valid[i] ; i++){
		if ( c == valid[i])
			return true;
	}
	
	return false;

}

void DatabaseNameToFileName( UInt8 *fileName, UInt8 *dbName)
{
	UInt8 c;
// Valid chars number, alphabet, space, * $ % ' - _ @ ~ ` ! ( ) ^ # & + , ; = [ ]
	while ( c = *dbName, c != 0){
		if ( IsKanji( dbName)){ // Kanji, copy it 
			*(fileName++)=*(dbName++);
			*(fileName++)=*(dbName++);
		} else if ( IsNumber( c) || IsAlphabet( c) || ( c == ' ') ){// Alphabet or Numeber , space , copy it
			*(fileName++)=*(dbName++);
		} else if ( IsValidFileNameChar( c ) ){
			*(fileName++)= *(dbName++);
		} else { // Non vaild, Let's URL Encode it
			*(fileName++)='%';
			*(fileName++)=IntToHex( c/ 16);
			*(fileName++)=IntToHex( c % 16);
			dbName++;
		}
	}
	*fileName = '\0';
	fileName++;
	

	return; 		
}

void
URLEncode(  char*dbName, char *fileName)
{
	UInt8 c;
// Valid chars number, alphabet, space, * $ % ' - _ @ ~ ` ! ( ) ^ # & + , ; = [ ]
	while ( c = *dbName, c != 0){
//		if ( IsNumber( c) || IsAlphabet( c) || ( c == ' ') ){// Alphabet or Numeber , space , copy it
		if ( IsNumber( c) || IsAlphabet( c) ){// Alphabet or Numeber , space , copy it
			*(fileName++)=*(dbName++);
		} else if ( IsValidFileNameChar( c ) ){
			*(fileName++)= *(dbName++);
		} else { // Non vaild, Let's URL Encode it
			*(fileName++)='%';
			*(fileName++)=IntToHex( c/ 16);
			*(fileName++)=IntToHex( c % 16);
			dbName++;
		}
	}
	*fileName = '\0';
	fileName++;
	
	return; 		
}


void
URLDecode(char *from, char *to )
{
	unsigned char *f = (unsigned char *)from;

	while ( *f ){
		if ( *f != '%'){
			*(to++) = *(f++);
		} else { // Decode %xx type				
			*(to++)= HexToInt(*(f+1), *(f+2));
			f += 3;
		}
					
	}

	*(to++)='\0'; // Terminate
}


int ByteCompare(char * from, char * to, int len)
{
	while ( len--)
		if ( *(from++) != *(to++))
			return 1;

	return 0;
}

void
GetFilenameExtension(char *fileName, char *extension)
{
	char * pos;
	
	pos = fileName + StrLen( fileName)-1; // Get last character;
	
	while( (*pos != '.') && (pos != fileName))
		pos--;
		
	if ( pos == fileName){
		StrCopy( extension,"");
		return;
	}
	
	StrCopy( extension, pos+1);
	return;
}

Boolean IsRunningOnPose(void)
{
	UInt32 temp;
	if (FtrGet ('pose', 0, &temp) == ftrErrNoSuchFeature)
	 return false;
	else
	 return true;
}

Boolean
DmFindDatabaseByName( char * p, UInt16 *cardp, LocalID *idp)
{
	int card = MemNumCards(); // How many card?
	int i;
	LocalID id;
	
	for ( i = 0; i < card;i++){
		id = DmFindDatabase( i, p);
		if ( id ){
			*cardp = i;
			*idp = id;
			return true;
		}
	}

	*cardp = 0;
	*idp = 0;
	return false;
}

DmOpenRef DmOpenDatabaseByName( char *name ,unsigned short mode)
{
	int card;
	LocalID id;
	
	for ( card= 0 ; card < MemNumCards();card++){
		id = DmFindDatabase( card, name);
		if ( id != 0 )
			return DmOpenDatabase( card, id, mode);
	}
	
	return 0; // not found
}


Err SetDefaultDepth(UInt32 newDefaultDepth)
{
         Err err = FtrSet(sysFtrCreator, sysFtrNumDisplayDepth, newDefaultDepth);
           if (!err)
              err = WinScreenMode(winScreenModeSetToDefaults, NULL, NULL, NULL, NULL);
           return err ;
}


Boolean BetaVersionExpired(DateTimeType start, DateTimeType end)
{
		UInt32 current,limit;
		DateTimeType limit_date;

		limit_date.minute = 0;
		limit_date.hour = 0;
		limit_date.day = end.day;
		limit_date.year= end.year;
		limit_date.month = end.month;

		limit = TimDateTimeToSeconds (&limit_date);
		limit += (long)2*24*60*60; // Add 2 days
		current =TimGetSeconds ();
	
		if ( current > limit ){
			return true;
		}

		limit_date.minute = 0;
		limit_date.hour = 0;
		limit_date.day = start.day;
		limit_date.month = start.month;
		limit_date.year= start.year;
		limit_date.weekDay= 0;		// Days since Sunday (0 to 6)

		limit = TimDateTimeToSeconds (&limit_date);
		
		if ( current < limit) // is old good days. it's expired
			return true;
		
		return false;
}

extern void GotoHome(void)
{
	EvtEnqueueKey( vchrLaunch , 0, commandKeyMask);
}

void CopyApplicationNameFromOpenDatabase( char *name, DmOpenRef db)
{
	MemHandle h;
	char* p;
	UInt16 cardNo;
	LocalID id;
	char db_name[33];
	UInt32 type, creator;

	DmOpenDatabaseInfo( db, &id, nil , nil, &cardNo,nil);
// Get DB infos
	DmDatabaseInfo ( cardNo, id, db_name, 0, 0, 0, 0, 0, 0, 0, 0, &type, &creator);
	 
	if (type == 'DAcc' ) { // It's DA. Copy DB name
		StrCopy( name, db_name);
	} else { // It's ap or panel. Lets's search for tAIN/1000 tAIN/1 and DB name
		h = DmGet1Resource ('tAIN', 1000); 
		if ( h == 0){ // no tAIN resource or it's DA. Let's get Database name
			h = DmGet1Resource ('tAIN', 1); 
		}
		
		if (h == 0){ // No resource. Copy db name	
			StrCopy( name, db_name);
		} else { 
			p = MemHandleLock(h);
			StrCopy( name,p);
		
			MemHandleUnlock(h);
			DmReleaseResource( h);
		}

/*		index = DmFindResource (db, 'tAIN', 1000, 0); 
		if ( index == -1){ // no tAIN resource or it's DA. Let's get Database name
			index = DmFindResource (db, 'tAIN', 1 ,0); 
		}
		
		if ( index == -1){ // No resource. Copy db name	
			StrCopy( name, db_name);
		} else { 
			h = DmGetResourceIndex (db, index) ;
			p = MemHandleLock(h);
			StrCopy( name,p);
		
			MemHandleUnlock(h);
			DmReleaseResource( h);
		}
*/
	}
	return;
}

void CopyApplicationNameFromTypeCreator( char *name,UInt32 type, UInt32 creator)
{
	DmOpenRef db;

	StrCopy( name, "Unknown");
	db = DmOpenDatabaseByTypeCreator( type, creator, dmModeReadOnly);
	if ( db){	
		CopyApplicationNameFromOpenDatabase( name, db);		
		DmCloseDatabase( db );
	}
	return;
}

void CopyApplicationNameFromDatabaseID( char *name, UInt16 cardNo, LocalID dbID)
{
	DmOpenRef db;

	StrCopy( name, "Unknown");
	db = DmOpenDatabase( cardNo, dbID, dmModeReadOnly);
	CopyApplicationNameFromOpenDatabase( name, db);		
	DmCloseDatabase( db );
	return;
}


void
UInt32ToString( char *str,UInt32 val)
{
	*str = (val >>24 );
	*(str+1) = (( val & 0x00ff0000 ) >> 16 );
	*(str+2) = (( val & 0x0000ff00 ) >> 8 );
	*(str+3) = ( val  & 0x000000ff );
	*(str+4) = 0;
}

UInt32
StringToUInt32(  char *str)
{
	UInt32 val;
	UInt8 *p = (UInt8 *)str;
	
	val = ( ((UInt32)( *p)) << 24 ) + ( ((UInt32)*(p+1)) << 16 ) + ( *(p+2) << 8 ) + ( *(p+3)) ;
	
	return val;
}


// For CLIE only

void
ChangeToHiResMode(UInt16 refNum)
{
	Err error;
	UInt32 width, height;

/********************************************/
/*CuOpen s*/
/********************************************/
	error = HROpen(refNum);
	if (error) {
		return;
	} else {
// Get Current Color depth
		WinScreenMode ( winScreenModeGet , &width,&height, 0 , 0);
//		depth = 8; /* (256 FJ[[h̏ꍇ)*/
		if ( (width != hrWidth) || ( height != hrHeight)){
			width = hrWidth;
			height = hrHeight;
			error = HRWinScreenMode ( refNum, winScreenModeSet, &width, &height, 0, 0 );
		}
	}
}

void ChangeToHiResIfAny(void)
{
	MachineFunctionType func;
	
	GetMachineFunctionality(&func);
	if ( func.supportHiResolution){
		UInt16 hrRefNum;
		Err err;
				
		err = SysLibFind(sonySysLibNameHR, &hrRefNum);
		if ( err ){
			ErrFatalDisplay("Can't get reference number for HiRes library");
		} else {
			ChangeToHiResMode(hrRefNum);
			HRClose (hrRefNum);
		}
	}
}

void
RevertToNormalMode(UInt16 refNum)
{
	Err error;

	error = HRWinScreenMode ( refNum, winScreenModeSetToDefaults , 0,0,0, 0 );

	HRClose( refNum);
}

FormActiveStateType *SaveFormState(void)
{
	FormActiveStateType *state;
	
	state = MemPtrNew( sizeof( FormActiveStateType ));
	FrmSaveActiveState( state);
	return state;
	
}

void RestoreFormState( FormActiveStateType*state)
{
	UInt32 time;
	
	FrmRestoreActiveState( state);
	MemPtrFree( state);

	return;
	
	time = TimGetTicks();
	
	while (1){
		EventType ev;
		if ( TimGetTicks() > time+1)
			break; // TIme out. Occures when DA have no forms.

		EvtGetEvent( &ev,0);
		if ( ev.eType == nilEvent)
			break;

		if (SysHandleEvent(&ev))
			continue;
			
		if ( ev.eType == winEnterEvent){
			break;
		}
		if ( ev.eType != winExitEvent)
			EvtAddEventToQueue( &ev);
	}
}

extern void ShowAboutForm( UInt16 id)
{
	FormType *save = FrmGetActiveForm();
	FormType *about = FrmInitForm ( id);
	FrmSetActiveForm( about);
	FrmDoDialog( about);
	FrmEraseForm (about);
	FrmDeleteForm( about);
	FrmSetActiveForm( save);
	
	return;
}