#include "MyUtilityResource.h"
#include <VFSMgr.h>

#ifndef __MyUtility__

#define __MyUtility__

#define MACHINE_FUNCTION_TYPE_VERSION 0x11

#ifndef IsSJIS1
	#define IsSJIS1(n)  ((((n)>0x80 ) && ( (n)<0xa0 )) || (((n)>0xdf) && ((n)<0xfe)) )
	#define IsSJIS2(n)  ((((n)>0x3f) && ((n)<0x7f)) || (((n)>0x7f) && ((n)<0xfd)) )
	#define IsASCII(n)   (((n)>0x1f)&&((n)<0x7f))
	#define IsKanji( n)  (( IsSJIS1( *(n) ) && IsSJIS2( *((n)+1)) ))
	#define IsNumber(n) ( ( (n)>='0'   ) && ( (n)<='9') )
	#define IsAlphabet(n)  (( (n)>='a'   ) && ( (n)<='z') || ( ( (n)>='A'   ) && ( (n)<='Z') ))
#endif

#ifndef nil
 #define nil 0
#endif

typedef struct {
	UInt16   structureVersion; // Hold MACHINE_FUNCTION_TYPE_VERSION 
	Boolean supportDynamicUI;
	Boolean supportBeam;
	Boolean supportFileStream;
	Boolean supportColorDrawing;
	Boolean supportDirectColor ; // 16bit etc.
	Boolean colorScreen; 
	Boolean supportHiResolution; // CLIE HiRes
	Boolean supportHandEraHiRes; // HandEra HiRes
	Boolean hasJogDial;		// CLie JOG Dial
	Boolean hasHandEraJog;	// Hand Era type JogDial
	Boolean hasTreoJog;	// Hand Treo 180 type JogDial
	Boolean hasGraffitiArea;	// Only false for treo 180
	Boolean has5WayNavigator;	// Palm Tungsten only( so far )
	Boolean hasOnly2Button; 	// Palm Zire series
	Boolean hasTreoKeyboard;	// Only true for treo 180
	Boolean hasTreoLid;			// Treo has switch related LID( 180/180g/270 not 90)
	Boolean hasTreoRadio;			// Treo has Radio Ability( 180/180g/270 not 90)
	Boolean supportClieSoftSilk;	// Only true for CLIE NR70
	Boolean supportHighDensity; 	// True for OS 5.0( earlier ?)
	Boolean hasBackButton;	// Clie Only
	Boolean hasJogAssist;	// Clie Only
	Boolean supportExternalMedia; // Supports MS or SD card by VFS Manager
	int		vfsManagerVersion;	  // 1 for CLIE 3.5 2 for PalmOS4.0
	int 	osVersion; // OS Version LIKE 0x351 or 0x200 
	UInt32	colorCapability; // shows how many colors supported
// Be care this might be false is Hires is supported!!!
	UInt32	screenWidth; // Screen Width in Standard Coordinate( Mostly 160 .)
	UInt32	screenHeight; // Screen Height in Standard Coordinate ( 160 or 240.)
	UInt32	screenWidthPixels; // Screen Width in Native Coordinates
	UInt32	screenHeightPixels; // Screen Height in Native Coordinates
	UInt16	nativeDensity;	// kCoordinatesStd, OneHalf or Double
} MachineFunctionType;

 
// External Variables
extern UInt32 OSVersion;
extern Boolean IsColorOS; // True if Supports Color IS
extern Boolean ExtMediaSupported;  // True if has VFSManager
extern Boolean JogDialSupported; // True if has JogDial

// External Functions

// Wrapper
extern MemPtr GetObjectPtrFromID(UInt16 objectID);
extern UInt16 GetObjectIndexFromID(UInt16 objectID);
extern UInt16 GetObjectIDFromIndex(UInt16 objectIndex);
extern MemPtr GetObjectPtrFromIndex(UInt16 objectIndex);
extern UInt16 GetObjectIDFromPtr( MemPtr objectPtr);
extern UInt16 GetObjectIndexFromPtr(MemPtr objectPtr);
extern void ShowObject( int id);
extern void HideObject( int id);
extern MemPtr GetObjectPtr(Int16 objectID);
extern void SetTextOnField(FieldType* field,Char* str);
extern void MyLibPresetPopTrigger( UInt16 triggerID, UInt16 listID, UInt16 selection);

// Utility
extern void GetMachineFunctionality(MachineFunctionType *func);
extern Boolean IsSonyDevice(void);
extern FormType *ShowWaitWindow(void);
extern void EraseWaitWindow( FormType *form);

// VFS Related
extern Err MyLibGetVolumeList(MemHandle vol_name_lists, UInt16 *num_volumes, MemHandle ref_num_lists );
extern Err MyLibGetDBNameFromMemoryCard( UInt16 volRefNum,char * full_path ,char *db_name);
extern Err MyLibVolumeWriteProteced( UInt16 volRefNum,Boolean *protected);
extern UInt16 MSGetNumVolumes();
extern Boolean SearchForFilesInDirectory(UInt16 volumeRefNum, FileRef *dir, char *path, UInt32 *iterator,FileInfoType *info);
extern void GetFilenameExtension(char *fileName, char *extension);
extern void DatabaseNameToFileName( UInt8 *fileName, UInt8 *dbName);
extern void FileNameToDatabaseName	(char * dbName, char *DocumentName);
extern Err MyLibMakeVFSFileList(UInt16 volRefNum, char *dir_path, UInt32 creator,UInt32 type, MemHandle listsH, UInt16 *num_files);
extern Boolean OsVersionCheck(void);
extern Boolean IsRunningOnPose(void);
extern Boolean DmFindDatabaseByName( char *db_name, UInt16 *cardp, LocalID *idp);
extern UInt16 MSGetVolumeRefNum( int number);
extern DmOpenRef DmOpenDatabaseByName( char *name ,unsigned short mode);
extern Err SetDefaultDepth(UInt32 newDefaultDepth);
extern Boolean BetaVersionExpired(DateTimeType start, DateTimeType end);
extern void GotoHome();
extern Boolean CheckMSFunctionality(void);
extern void CopyApplicationNameFromTypeCreator( char* name,UInt32 type, UInt32 creator);
extern void CopyApplicationNameFromDatabaseID( char  *name, UInt16 cardNo, LocalID dbID);
extern void UInt32ToString( char *str,UInt32 val);
extern UInt32 StringToUInt32( char *str);
extern void CheckCreate( UInt16 volRefNum, char *path);
extern int HexToInt(char b1, char b2);
extern FormActiveStateType *SaveFormState(void);
extern void RestoreFormState( FormActiveStateType*state);
extern void URLEncode(  char*dbName, char *fileName);
extern void URLDecode(char *from, char *to );
extern void ShowAboutForm( UInt16 form_id);

//  CLIE HiRes related
extern void ChangeToHiResMode(UInt16 refNum);
extern void RevertToNormalMode(UInt16 refNum);
extern void ChangeToHiResIfAny(void);

#endif
